@@ -1,7 +1,8 @@
  # vim: set sw=4 expandtab :
  #
- # Copyright 2004 Apache Software Foundation 
- # 
+ # Copyright (C) 2000, 2001, 2013, 2024 Gregory Trubetskoy
+ # Copyright (C) 2002, 2003, 2004, 2005, 2006, 2007 Apache Software Foundation
+ #
  # Licensed under the Apache License, Version 2.0 (the "License"); you
  # may not use this file except in compliance with the License.  You
  # may obtain a copy of the License at
@@ -16,7 +17,6 @@
  #
  # Originally developed by Gregory Trubetskoy.
  #
- # $Id: Cookie.py 472053 2006-11-07 10:11:01Z grahamd $
 
 """
 
@@ -41,13 +41,14 @@ Max-Age and Version) could be a waste of cookie space...
 
 """
 
+import sys
 import time
 import re
 import hmac
 import marshal
 import base64
 
-# import apache
+PY2 = sys.version[0] == '2'
 
 class CookieError(Exception):
     pass
@@ -58,7 +59,7 @@ class metaCookie(type):
 
         _valid_attr = (
             "version", "path", "domain", "secure",
-            "comment", "expires", "max_age",
+            "comment", "expires", "max_age", "samesite",
             # RFC 2965
             "commentURL", "discard", "port",
             # Microsoft Extension
@@ -73,6 +74,10 @@ class metaCookie(type):
         clsdict["_valid_attr"] = _valid_attr
         clsdict["__slots__"] = __slots__
 
+        return type.__new__(cls, clsname, bases, clsdict)
+
+    def __init__(cls, clsname, bases, clsdict):
+
         def set_expires(self, value):
 
             if type(value) == type(""):
@@ -81,7 +86,7 @@ class metaCookie(type):
                 try:
                     t = time.strptime(value, "%a, %d-%b-%Y %H:%M:%S GMT")
                 except ValueError:
-                    raise ValueError, "Invalid expires time: %s" % value
+                    raise ValueError("Invalid expires time: %s" % value)
                 t = time.mktime(t)
             else:
                 # otherwise assume it's a number
@@ -95,19 +100,19 @@ class metaCookie(type):
         def get_expires(self):
             return self._expires
 
-        clsdict["expires"] = property(fget=get_expires, fset=set_expires)
+        cls.expires = property(fget=get_expires, fset=set_expires)
 
-        return type.__new__(cls, clsname, bases, clsdict)
+# metaclass= workaround, see
+# http://mikewatkins.ca/2008/11/29/python-2-and-3-metaclasses/#using-the-metaclass-in-python-2-x-and-3-x
+_metaCookie = metaCookie('Cookie', (object, ), {})
 
-class Cookie(object):
+class Cookie(_metaCookie):
     """
     This class implements the basic Cookie functionality. Note that
     unlike the Python Standard Library Cookie class, this class represents
     a single cookie (not a list of Morsels).
     """
 
-    __metaclass__ = metaCookie
-
     DOWNGRADE = 0
     IGNORE = 1
     EXCEPTION = 3
@@ -129,7 +134,7 @@ class Cookie(object):
         """
         This constructor takes at least a name and value as the
         arguments, as well as optionally any of allowed cookie attributes
-        as defined in the existing cookie standards. 
+        as defined in the existing cookie standards.
         """
         self.name, self.value = name, value
 
@@ -152,7 +157,7 @@ class Cookie(object):
         dictate this. This is because doing so seems to confuse most
         browsers out there.
         """
-        
+
         result = ["%s=%s" % (self.name, self.value)]
         for name in self._valid_attr:
             if hasattr(self, name):
@@ -161,11 +166,11 @@ class Cookie(object):
                 else:
                     result.append("%s=%s" % (name, getattr(self, name)))
         return "; ".join(result)
-    
+
     def __repr__(self):
         return '<%s: %s>' % (self.__class__.__name__,
                                 str(self))
-    
+
 
 class SignedCookie(Cookie):
     """
@@ -210,13 +215,16 @@ class SignedCookie(Cookie):
 
     def hexdigest(self, str):
         if not self.__data__["secret"]:
-            raise CookieError, "Cannot sign without a secret"
+            raise CookieError("Cannot sign without a secret")
         _hmac = hmac.new(self.__data__["secret"], self.name)
         _hmac.update(str)
-        return _hmac.hexdigest()
+        if PY2:
+            return _hmac.hexdigest()
+        else:
+            return _hmac.hexdigest().decode()
 
     def __str__(self):
-        
+
         result = ["%s=%s%s" % (self.name, self.hexdigest(self.value),
                                self.value)]
         for name in self._valid_attr:
@@ -238,7 +246,7 @@ class SignedCookie(Cookie):
             self.value = val
             self.__data__["secret"] = secret
         else:
-            raise CookieError, "Incorrectly Signed Cookie: %s=%s" % (self.name, self.value)
+            raise CookieError("Incorrectly Signed Cookie: %s=%s" % (self.name, self.value))
 
 
 class MarshalCookie(SignedCookie):
@@ -284,7 +292,7 @@ class MarshalCookie(SignedCookie):
     parse = classmethod(parse)
 
     def __str__(self):
-        
+
         m = base64.encodestring(marshal.dumps(self.value))
         # on long cookies, the base64 encoding can contain multiple lines
         # separated by \n or \r\n
@@ -306,12 +314,12 @@ class MarshalCookie(SignedCookie):
         try:
             data = base64.decodestring(self.value)
         except:
-            raise CookieError, "Cannot base64 Decode Cookie: %s=%s" % (self.name, self.value)
+            raise CookieError("Cannot base64 Decode Cookie: %s=%s" % (self.name, self.value))
 
         try:
             self.value = marshal.loads(data)
         except (EOFError, ValueError, TypeError):
-            raise CookieError, "Cannot Unmarshal Cookie: %s=%s" % (self.name, self.value)
+            raise CookieError("Cannot Unmarshal Cookie: %s=%s" % (self.name, self.value))
 
 
 # This is a simplified and in some places corrected
@@ -344,7 +352,7 @@ def _parse_cookie(str, Class, names=None):
 
         # We just ditch the cookies names which start with a dollar sign since
         # those are in fact RFC2965 cookies attributes. See bug [#MODPYTHON-3].
-        if key[0]!='$' and names is None or key in names:
+        if key[0]!='$' and (names is None or key in names):
             result[key] = Class(key, val)
 
     return result
@@ -360,8 +368,8 @@ def add_cookie(req, cookie, value="", **kw):
 
         # make a cookie
         cookie = Cookie(cookie, value, **kw)
-        
-    if not req.headers_out.has_key("Set-Cookie"):
+
+    if "Set-Cookie" not in req.headers_out:
         req.headers_out.add("Cache-Control", 'no-cache="set-cookie"')
 
     req.headers_out.add("Set-Cookie", str(cookie))
@@ -372,8 +380,8 @@ def get_cookies(req, Class=Cookie, **kw):
     a Cookie class. The class must be one of the classes from
     this module.
     """
-    
-    if not req.headers_in.has_key("cookie"):
+
+    if "cookie" not in req.headers_in:
         return {}
 
     cookies = req.headers_in["cookie"]
@@ -384,5 +392,5 @@ def get_cookies(req, Class=Cookie, **kw):
 
 def get_cookie(req, name, Class=Cookie, **kw):
     cookies = get_cookies(req, Class, names=[name], **kw)
-    if cookies.has_key(name):
+    if name in cookies:
         return cookies[name]
