@@ -1,6 +1,7 @@
  #
- # Copyright 2004 Apache Software Foundation 
- # 
+ # Copyright (C) 2000, 2001, 2013, 2024 Gregory Trubetskoy
+ # Copyright (C) 2002, 2003, 2004, 2005, 2006, 2007 Apache Software Foundation
+ #
  # Licensed under the Apache License, Version 2.0 (the "License"); you
  # may not use this file except in compliance with the License.  You
  # may obtain a copy of the License at
@@ -15,10 +16,17 @@
  #
  # Originally developed by Gregory Trubetskoy.
  #
- # $Id: __init__.py 501044 2007-01-29 13:39:12Z jgallacher $
 
 __all__ = ["apache", "cgihandler", "psp",
-           "publisher", "util", "python22"]
+           "publisher", "util", "python22", "version"]
 
-version = "3.3.1"
+# This is used by mod_python.c to make sure the version of C
+# code matches the Python code.
+from . import version
+mp_version = version.version
 
+try:
+    # it's possible for mod_python to be imported outside httpd, e.g. to use
+    # httpdconf, so we fail silently
+    from . import apache
+except: pass
