@@ -1,6 +1,7 @@
  # vim: set sw=4 expandtab :
  #
- # Copyright 2004 Apache Software Foundation
+ # Copyright (C) 2000, 2001, 2013, 2024 Gregory Trubetskoy
+ # Copyright (C) 2002, 2003, 2004, 2005, 2006, 2007 Apache Software Foundation
  #
  # Licensed under the Apache License, Version 2.0 (the "License"); you
  # may not use this file except in compliance with the License.  You
@@ -16,7 +17,6 @@
  #
  # Originally developed by Gregory Trubetskoy.
  #
- # $Id: apache.py 468216 2006-10-27 00:54:12Z grahamd $
 
 import sys
 import traceback
@@ -24,10 +24,14 @@ import time
 import os
 import pdb
 import stat
-import imp
+import inspect
+import importlib
 import types
-import cgi
 import _apache
+try:
+    from html import escape
+except:
+    from cgi import escape
 
 try:
     import threading
@@ -40,7 +44,7 @@ _path_cache_lock = threading.Lock()
 
 _result_warning = """Handler has returned result or raised SERVER_RETURN
 exception with argument having non integer type. Type of value returned
-was %s, whereas expected """ + str(types.IntType) + "."
+was %s, whereas expected """ + str(int) + "."
 
 class CallBack:
     """
@@ -69,8 +73,9 @@ class CallBack:
     def ConnectionDispatch(self, conn):
 
         # config
-        config = conn.base_server.get_config()
-        debug = int(config.get("PythonDebug", 0))
+        config, debug  = conn.base_server.get_config(), False
+        if "PythonDebug" in config:
+            debug = config["PythonDebug"] == "1"
 
         try:
 
@@ -81,39 +86,43 @@ class CallBack:
             module_name = l[0]
             if len(l) == 1:
                 # no oject, provide default
-                object_str = "connectionhandler"
+                obj_str = "connectionhandler"
             else:
-                object_str = l[1]
+                obj_str = l[1]
 
             # evaluate pythonpath and set sys.path to
             # resulting value if not already done
 
-            if config.has_key("PythonPath"): 
-                _path_cache_lock.acquire() 
-                try: 
-                    pathstring = config["PythonPath"] 
-                    if not _path_cache.has_key(pathstring): 
-                        newpath = eval(pathstring) 
-                        _path_cache[pathstring] = None 
-                        sys.path[:] = newpath 
-                finally: 
+            if "PythonPath" in config:
+                _path_cache_lock.acquire()
+                try:
+                    pathstring = config["PythonPath"]
+                    if pathstring not in _path_cache:
+                        newpath = eval(pathstring)
+                        _path_cache[pathstring] = None
+                        sys.path[:] = newpath
+                finally:
                     _path_cache_lock.release()
 
             # import module
+            autoreload = True
+            if "PythonAutoReload" in config:
+                autoreload = config["PythonAutoReload"] == "1"
             module = import_module(module_name,
-                                   autoreload=int(config.get("PythonAutoReload", 1)),
+                                   autoreload=autoreload,
                                    log=debug)
             # find the object
-            object = resolve_object(module, object_str,
+            obj = resolve_object(module, obj_str,
                                     arg=conn, silent=0)
 
             # Only permit debugging using pdb if Apache has
             # actually been started in single process mode.
 
-            pdb_debug = int(config.get("PythonEnablePdb", "0"))
-            one_process = exists_config_define("ONE_PROCESS")
+            pdb_debug = False
+            if "PythonEnablePdb" in config:
+                pdb_debug = config["PythonEnablePdb"] == "1"
 
-            if pdb_debug and one_process:
+            if pdb_debug and exists_config_define("ONE_PROCESS"):
 
                 # Don't use pdb.runcall() as it results in
                 # a bogus 'None' response when pdb session
@@ -127,30 +136,20 @@ class CallBack:
                 sys.settrace(debugger.trace_dispatch)
 
                 try:
-                    result = object(conn)
+                    result = obj(conn)
 
                 finally:
                     debugger.quitting = 1
                     sys.settrace(None)
 
             else:
-                result = object(conn)
+                result = obj(conn)
 
-            assert (type(result) == type(int())), \
+            assert (result.__class__ is int), \
                    "ConnectionHandler '%s' returned invalid return code." % handler
 
-        except PROG_TRACEBACK, traceblock:
-            # Program run-time error
-            try:
-                (etype, value, traceback) = traceblock
-                result = self.ReportError(etype, value, traceback, srv=conn.base_server,
-                                          phase="ConnectionHandler",
-                                          hname=handler, debug=debug)
-            finally:
-                traceback = None
-
         except:
-            # Any other rerror (usually parsing)
+            # Error (usually parsing)
             try:
                 exc_type, exc_value, exc_traceback = sys.exc_info()
                 result = self.ReportError(exc_type, exc_value, exc_traceback, srv=conn.base_server,
@@ -161,66 +160,72 @@ class CallBack:
 
         return result
 
-    def FilterDispatch(self, filter):
-
-        req = filter.req
+    def FilterDispatch(self, fltr):
+        req = fltr.req
 
         # config
-        config = req.get_config()
-        debug = int(config.get("PythonDebug", 0))
+        config, debug  = req.get_config(), False
+        if "PythonDebug" in config:
+            debug = config["PythonDebug"] == "1"
 
         try:
 
+            # split module::handler
+            l = fltr.handler.split('::', 1)
+            module_name = l[0]
+            if len(l) == 1:
+                # no oject, provide default
+                if fltr.is_input:
+                    obj_str = "inputfilter"
+                else:
+                    obj_str = "outputfilter"
+            else:
+                obj_str = l[1]
+
             # add the directory to pythonpath if
             # not there yet, or evaluate pythonpath
             # and set sys.path to resulting value
             # if not already done
 
-            if config.has_key("PythonPath"): 
-                _path_cache_lock.acquire() 
-                try: 
-                    pathstring = config["PythonPath"] 
-                    if not _path_cache.has_key(pathstring): 
-                        newpath = eval(pathstring) 
-                        _path_cache[pathstring] = None 
-                        sys.path[:] = newpath 
-                finally: 
+            if "PythonPath" in config:
+                _path_cache_lock.acquire()
+                try:
+                    pathstring = config["PythonPath"]
+                    if pathstring not in _path_cache:
+                        newpath = eval(pathstring)
+                        _path_cache[pathstring] = None
+                        sys.path[:] = newpath
+                finally:
                     _path_cache_lock.release()
             else:
-                if filter.dir and (filter.dir not in sys.path):
-                    sys.path[:0] = [filter.dir]
+                if fltr.dir:
+                    _path_cache_lock.acquire()
+                    try:
+                        if fltr.dir not in sys.path:
+                            sys.path[:0] = [fltr.dir]
+                    finally:
+                        _path_cache_lock.release()
 
-            if not callable(filter.handler):
+            # import module
+            autoreload = True
+            if "PythonAutoReload" in config:
+                autoreload = config["PythonAutoReload"] == "1"
+            module = import_module(module_name,
+                                   autoreload=autoreload,
+                                   log=debug)
 
-                # split module::handler
-                l = filter.handler.split('::', 1)
-                module_name = l[0]
-                if len(l) == 1:
-                    # no oject, provide default
-                    if filter.is_input:
-                        object_str = "inputfilter"
-                    else:
-                        object_str = "outputfilter"
-                else:
-                    object_str = l[1]
-
-                # import module
-                module = import_module(module_name,
-                                       autoreload=int(config.get("PythonAutoReload", 1)),
-                                       log=debug)
-
-                # find the object
-                object = resolve_object(module, object_str, arg=filter, silent=0)
-            else:
-                object = filter.handler
+            # find the object
+            obj = resolve_object(module, obj_str,
+                                    arg=fltr, silent=0)
 
             # Only permit debugging using pdb if Apache has
             # actually been started in single process mode.
 
-            pdb_debug = int(config.get("PythonEnablePdb", "0"))
-            one_process = exists_config_define("ONE_PROCESS")
+            pdb_debug = False
+            if "PythonEnablePdb" in config:
+                pdb_debug = config["PythonEnablePdb"] == "1"
 
-            if pdb_debug and one_process:
+            if pdb_debug and exists_config_define("ONE_PROCESS"):
 
                 # Don't use pdb.runcall() as it results in
                 # a bogus 'None' response when pdb session
@@ -234,22 +239,22 @@ class CallBack:
                 sys.settrace(debugger.trace_dispatch)
 
                 try:
-                    result = object(filter)
+                    result = obj(fltr)
 
                 finally:
                     debugger.quitting = 1
                     sys.settrace(None)
 
             else:
-                result = object(filter)
+                result = obj(fltr)
 
             # always flush the filter. without a FLUSH or EOS bucket,
             # the content is never written to the network.
             # XXX an alternative is to tell the user to flush() always
-            if not filter.closed: 
-                filter.flush()
+            if not fltr.closed:
+                fltr.flush()
 
-        except SERVER_RETURN, value:
+        except SERVER_RETURN as value:
             # SERVER_RETURN indicates a non-local abort from below
             # with value as (result, status) or (result, None) or result
             try:
@@ -260,34 +265,23 @@ class CallBack:
                 else:
                     result = value.args[0]
 
-                if type(result) != type(7):
+                if result.__class__ is not int:
                     s = "Value raised with SERVER_RETURN is invalid. It is a "
-                    s = s + "%s, but it must be a tuple or an int." % type(result)
-                    _apache.log_error(s, APLOG_NOERRNO|APLOG_ERR, req.server)
+                    s = s + "%s, but it must be a tuple or an int." % result.__class__
+                    _apache.log_error(s, APLOG_ERR, req.server)
 
                     return
 
             except:
                 pass
 
-        except PROG_TRACEBACK, traceblock:
-            # Program run-time error
-            try:
-                (etype, value, traceback) = traceblock
-                filter.disable()
-                result = self.ReportError(etype, value, traceback, req=req, filter=filter,
-                                          phase="Filter: " + filter.name,
-                                          hname=filter.handler, debug=debug)
-            finally:
-                traceback = None
-
         except:
-            # Any other rerror (usually parsing)
+            # Error (usually parsing)
             try:
                 exc_type, exc_value, exc_traceback = sys.exc_info()
-                filter.disable()
-                result = self.ReportError(exc_type, exc_value, exc_traceback, req=req, filter=filter,
-                                          phase=filter.name, hname=filter.handler,
+                fltr.disable()
+                result = self.ReportError(exc_type, exc_value, exc_traceback, req=req, filter=fltr,
+                                          phase=fltr.name, hname=fltr.handler,
                                           debug=debug)
             finally:
                 exc_traceback = None
@@ -303,75 +297,88 @@ class CallBack:
         result = HTTP_INTERNAL_SERVER_ERROR
 
         # config
-        config = req.get_config()
-        debug = int(config.get("PythonDebug", 0))
+        config, debug = req.get_config(), False
+        if "PythonDebug" in config:
+            debug = config["PythonDebug"] == "1"
 
-        default_object_str = req.phase[len("python"):].lower()
+        default_obj_str = _phase_handler_names[req.phase]
 
         # Lookup expected status values that allow us to
         # continue when multiple handlers exist.
-
-        expected = _status_values[default_object_str]
+        expected = _status_values[default_obj_str]
 
         try:
             hlist = req.hlist
 
             while hlist.handler is not None:
 
+                # split module::handler
+                l = hlist.handler.split('::', 1)
+
+                module_name = l[0]
+                if len(l) == 1:
+                    # no object, provide default
+                    obj_str = default_obj_str
+                else:
+                    obj_str = l[1]
+
                 # add the directory to pythonpath if
                 # not there yet, or evaluate pythonpath
                 # and set sys.path to resulting value
                 # if not already done
 
-                if config.has_key("PythonPath"): 
-                    _path_cache_lock.acquire() 
-                    try: 
-                        pathstring = config["PythonPath"] 
-                        if not _path_cache.has_key(pathstring): 
-                            newpath = eval(pathstring) 
-                            _path_cache[pathstring] = None 
-                            sys.path[:] = newpath 
-                    finally: 
+                if "PythonPath" in config:
+                    _path_cache_lock.acquire()
+                    try:
+                        pathstring = config["PythonPath"]
+                        if pathstring not in _path_cache:
+                            newpath = eval(pathstring)
+                            _path_cache[pathstring] = None
+                            sys.path[:] = newpath
+                    finally:
                         _path_cache_lock.release()
                 else:
-                    dir = hlist.directory
-                    if dir and (dir not in sys.path):
-                        sys.path[:0] = [dir]
+                    if not hlist.is_location:
+                        directory = hlist.directory
+                        if directory:
+                            _path_cache_lock.acquire()
+                            try:
+                                if directory not in sys.path:
+                                    sys.path[:0] = [directory]
+                            finally:
+                                _path_cache_lock.release()
 
-                if not callable(hlist.handler):
-
-                    # split module::handler
-                    l = hlist.handler.split('::', 1)
-
-                    module_name = l[0]
-                    if len(l) == 1:
-                        # no object, provide default
-                        object_str = default_object_str
-                    else:
-                        object_str = l[1]
-
-                    # import module
-                    module = import_module(module_name,
-                                           autoreload=int(config.get("PythonAutoReload", 1)),
-                                           log=debug)
-
-                    # find the object
-                    object = resolve_object(module, object_str,
-                                            arg=req, silent=hlist.silent)
+                # import module
+                autoreload = True
+                if "PythonAutoReload" in config:
+                    autoreload = config["PythonAutoReload"] == "1"
+                module = import_module(module_name,
+                                       autoreload=autoreload,
+                                       log=debug)
 
+                # find the object
+                if '.' not in obj_str: # this is an optimization
+                    try:
+                        obj = module.__dict__[obj_str]
+                    except:
+                        if not hlist.silent:
+                            s = "module '%s' contains no '%s'" % (module.__file__, obj_str)
+                            raise AttributeError(s)
                 else:
-                    object = hlist.handler
+                    obj = resolve_object(module, obj_str,
+                                         arg=req, silent=hlist.silent)
 
-                if not hlist.silent or object is not None:
+                if not hlist.silent or obj is not None:
 
                     try:
                         # Only permit debugging using pdb if Apache has
                         # actually been started in single process mode.
 
-                        pdb_debug = int(config.get("PythonEnablePdb", "0"))
-                        one_process = exists_config_define("ONE_PROCESS")
+                        pdb_debug = False
+                        if "PythonEnablePdb" in config:
+                            pdb_debug = config["PythonEnablePdb"] == "1"
 
-                        if pdb_debug and one_process:
+                        if pdb_debug and exists_config_define("ONE_PROCESS"):
 
                             # Don't use pdb.runcall() as it results in
                             # a bogus 'None' response when pdb session
@@ -385,16 +392,16 @@ class CallBack:
                             sys.settrace(debugger.trace_dispatch)
 
                             try:
-                                result = object(req)
+                                result = obj(req)
 
                             finally:
                                 debugger.quitting = 1
                                 sys.settrace(None)
 
                         else:
-                            result = object(req)
+                            result = obj(req)
 
-                    except SERVER_RETURN, value:
+                    except SERVER_RETURN as value:
 
                         # The SERVER_RETURN exception type when raised
                         # otherwise indicates an abort from below with
@@ -408,8 +415,8 @@ class CallBack:
                         else:
                             result = value.args[0]
 
-                    assert (type(result) == types.IntType), \
-                            _result_warning % type(result)
+                    assert (result.__class__ is int), \
+                            _result_warning % result.__class__
 
                     # stop cycling through handlers
                     if result not in expected:
@@ -425,18 +432,8 @@ class CallBack:
 
                 hlist.next()
 
-        except PROG_TRACEBACK, traceblock:
-            # Program run-time error
-            try:
-                (etype, value, traceback) = traceblock
-                result = self.ReportError(etype, value, traceback, req=req,
-                                          phase=req.phase, hname=hlist.handler,
-                                          debug=debug)
-            finally:
-                traceback = None
-
         except:
-            # Any other rerror (usually parsing)
+            # Error (usually parsing)
             try:
                 exc_type, exc_value, exc_traceback = sys.exc_info()
                 result = self.ReportError(exc_type, exc_value, exc_traceback, req=req,
@@ -446,64 +443,68 @@ class CallBack:
 
         return result
 
-    def IncludeDispatch(self, filter, tag, code):
+    def IncludeDispatch(self, fltr, tag, code):
 
         try:
-            config = filter.req.get_config()
-            debug = int(config.get("PythonDebug", 0))
+            # config
+            config, debug = fltr.req.get_config(), False
+            if "PythonDebug" in config:
+                debug = config["PythonDebug"] == "1"
 
-            if not hasattr(filter.req,"ssi_globals"):
-                filter.req.ssi_globals = {}
+            if not hasattr(fltr.req,"ssi_globals"):
+                fltr.req.ssi_globals = {}
 
-            filter.req.ssi_globals["filter"] = filter
-
-            filter.req.ssi_globals["__file__"] = filter.req.filename
+            fltr.req.ssi_globals["filter"] = fltr
+            fltr.req.ssi_globals["__file__"] = fltr.req.filename
 
             code = code.replace('\r\n', '\n').rstrip()
 
             if tag == 'eval':
-                result = eval(code, filter.req.ssi_globals)
+                result = eval(code, fltr.req.ssi_globals)
                 if result is not None:
-                    filter.write(str(result))
+                    fltr.write(str(result))
             elif tag == 'exec':
-                exec(code, filter.req.ssi_globals)
+                exec(code, fltr.req.ssi_globals)
 
-            filter.flush()
+            fltr.flush()
 
         except:
             try:
                 exc_type, exc_value, exc_traceback = sys.exc_info()
+                fltr.disable()
                 result = self.ReportError(exc_type, exc_value, exc_traceback,
-                                          filter=filter, phase=filter.name,
-                                          hname=filter.req.filename,
+                                          req=fltr.req, filter=fltr,
+                                          phase=fltr.name,
+                                          hname=fltr.req.filename,
                                           debug=debug)
             finally:
                 exc_traceback = None
 
             raise
 
-        filter.req.ssi_globals["filter"] = None
+        fltr.req.ssi_globals["filter"] = None
 
         return OK
 
     def ImportDispatch(self, name):
 
-        config = main_server.get_config()
-
-        debug = int(config.get("PythonDebug", "0"))
+        # config
+        config, debug = main_server.get_config(), False
+        if "PythonDebug" in config:
+            debug = config["PythonDebug"] == "1"
 
         # evaluate pythonpath and set sys.path to
         # resulting value if not already done
 
-        if config.has_key("PythonPath"): 
-            _path_cache_lock.acquire() 
-            try: 
-                pathstring = config["PythonPath"] 
-                if not _path_cache.has_key(pathstring): 
-                    newpath = eval(pathstring) 
-                    _path_cache[pathstring] = None 
-                    sys.path[:] = newpath 
-            finally: 
+        if "PythonPath" in config:
+            _path_cache_lock.acquire()
+            try:
+                pathstring = config["PythonPath"]
+                if pathstring not in _path_cache:
+                    newpath = eval(pathstring)
+                    _path_cache[pathstring] = None
+                    sys.path[:] = newpath
+            finally:
                 _path_cache_lock.release()
 
         # split module::function
@@ -539,9 +540,9 @@ class CallBack:
                 for e in traceback.format_exception(etype, evalue, etb):
                     s = "%s %s: %s" % (phase, hname, e[:-1])
                     if req:
-                        req.log_error(s, APLOG_NOERRNO|APLOG_ERR)
+                        req.log_error(s, APLOG_ERR)
                     else:
-                        _apache.log_error(s, APLOG_NOERRNO|APLOG_ERR, srv)
+                        _apache.log_error(s, APLOG_ERR, srv)
 
                 if not debug or not req:
                     return HTTP_INTERNAL_SERVER_ERROR
@@ -552,7 +553,7 @@ class CallBack:
 
                     s = '\n<pre>\nMod_python error: "%s %s"\n\n' % (phase, hname)
                     for e in traceback.format_exception(etype, evalue, etb):
-                        s = s + cgi.escape(e) + '\n'
+                        s = s + escape(e) + '\n'
                     s = s + "</pre>\n"
 
                     if filter:
@@ -581,73 +582,68 @@ def import_module(module_name, autoreload=1, log=0, path=None):
 
     # nlehuen: this is a big lock, we'll have to refine it later to get better performance.
     # For now, we'll concentrate on thread-safety.
-    imp.acquire_lock()
-    try:
-        # (Re)import
-        if sys.modules.has_key(module_name):
+    # (Re)import
+    if module_name in sys.modules:
 
-            # The module has been imported already
-            module = sys.modules[module_name]
+        # The module has been imported already
+        module = sys.modules[module_name]
+        oldmtime, mtime  = 0, 0
+
+        if autoreload:
 
             # but is it in the path?
-            file = module.__dict__.get("__file__")
+            try:
+                file = module.__dict__["__file__"]
+            except KeyError:
+                file = None
 
             # the "and not" part of this condition is to prevent execution
             # of arbitrary already imported modules, such as os. The
             # reason we use startswith as opposed to exact match is that
             # modules inside packages are actually in subdirectories.
 
-            if not file or (path and not filter(file.startswith, path)):
+            if not file or (path and not list(filter(file.startswith, path))):
                 # there is a script by this name already imported, but it's in
                 # a different directory, therefore it's a different script
-                mtime, oldmtime = 0, -1
-            elif autoreload:
-                oldmtime = module.__dict__.get("__mtime__", 0)
-                mtime = module_mtime(module)
+                mtime, oldmtime = 0, -1 # trigger import
             else:
-                mtime, oldmtime = 0, 0
-
-        else:
-            mtime, oldmtime = 0, -1
-
-        if mtime != oldmtime:
-
-            # Import the module
-            if log:
-                if path:
-                    s = "mod_python: (Re)importing module '%s' with path set to '%s'" % (module_name, path)
-                else:
-                    s = "mod_python: (Re)importing module '%s'" % module_name
-                _apache.log_error(s, APLOG_NOERRNO|APLOG_NOTICE)
-
-            parent = None
-            parts = module_name.split('.')
-            for i in range(len(parts)):
-                f, p, d = imp.find_module(parts[i], path)
                 try:
-                    mname = ".".join(parts[:i+1])
-                    module = imp.load_module(mname, f, p, d)
-                    if parent:
-                        setattr(parent,parts[i],module)
-                    parent = module
-                finally:
-                    if f: f.close()
-                if hasattr(module, "__path__"):
-                    path = module.__path__
+                    last_check = module.__dict__["__mtime_check__"]
+                except KeyError:
+                    last_check = 0
 
-            if mtime == 0:
-                mtime = module_mtime(module)
+                if (time.time() - last_check) > 1:
+                    oldmtime = module.__dict__.get("__mtime__", 0)
+                    mtime = module_mtime(module)
+        else:
+            pass
+    else:
+        mtime, oldmtime = 0, -1
 
-            module.__mtime__ = mtime
+    if mtime != oldmtime:
 
-        return module
-    finally:
-        imp.release_lock()
+        # Import the module
+        if log:
+            if path:
+                s = "mod_python: (Re)importing module '%s' with path set to '%s'" % (module_name, path)
+            else:
+                s = "mod_python: (Re)importing module '%s'" % module_name
+            _apache.log_error(s, APLOG_NOTICE)
+
+        parent = None
+        module = importlib.import_module(module_name)
+
+        if mtime == 0:
+            mtime = module_mtime(module)
+
+        module.__mtime__ = mtime
+
+    return module
 
 def module_mtime(module):
     """Get modification time of module"""
     mtime = 0
-    if module.__dict__.has_key("__file__"):
+    if "__file__" in module.__dict__:
 
         filepath = module.__file__
 
@@ -662,11 +658,12 @@ def module_mtime(module):
             if os.path.exists(filepath[:-1]) :
                 mtime = max(mtime, os.path.getmtime(filepath[:-1]))
 
+            module.__dict__["__mtime_check__"] = time.time()
         except OSError: pass
 
     return mtime
 
-def resolve_object(module, object_str, arg=None, silent=0):
+def resolve_object(module, obj_str, arg=None, silent=0):
     """
     This function traverses the objects separated by .
     (period) to find the last one we're looking for:
@@ -681,7 +678,7 @@ def resolve_object(module, object_str, arg=None, silent=0):
 
     obj = module
 
-    for obj_str in  object_str.split('.'):
+    for obj_str in obj_str.split('.'):
 
         parent = obj
 
@@ -693,13 +690,13 @@ def resolve_object(module, object_str, arg=None, silent=0):
         if obj == module and not hasattr(module, obj_str):
             if hasattr(module, "__file__"):
                 s = "module '%s' contains no '%s'" % (module.__file__, obj_str)
-                raise AttributeError, s
+                raise AttributeError(s)
 
         obj = getattr(obj, obj_str)
 
-        if hasattr(obj, "im_self") and not obj.im_self:
-            # this is an unbound method, its class
-            # needs to be instantiated
+        if inspect.isclass(parent):
+            # The parent of this object is a class,
+            # it needs to be instantiated
             instance = parent(arg)
             obj = getattr(instance, obj_str)
 
@@ -712,7 +709,7 @@ def build_cgi_env(req):
     http://hoohoo.ncsa.uiuc.edu/cgi/env.html
     """
 
-    req.add_common_vars()
+    req.add_cgi_vars()
     env = req.subprocess_env.copy()
 
     if req.path_info and len(req.path_info) > 0:
@@ -723,12 +720,12 @@ def build_cgi_env(req):
     env["GATEWAY_INTERFACE"] = "Python-CGI/1.1"
 
     # you may want to comment this out for better security
-    if req.headers_in.has_key("authorization"):
+    if "authorization" in req.headers_in:
         env["HTTP_AUTHORIZATION"] = req.headers_in["authorization"]
 
     return env
 
-class NullIO:
+class NullIO(object):
     """ Abstract IO
     """
     def tell(self): return 0
@@ -741,6 +738,7 @@ class NullIO:
     def isatty(self): return 0
     def flush(self): pass
     def close(self): pass
+    def detach(self): pass
     def seek(self, pos, mode = 0): pass
 
 class CGIStdin(NullIO):
@@ -777,7 +775,7 @@ class CGIStdin(NullIO):
 
     def readlines(self):
         s = (self.buf + self.read()).split('\n')
-        return map(lambda s: s + '\n', s)
+        return [s + '\n' for s in s]
 
     def readline(self, n = -1):
 
@@ -895,7 +893,7 @@ def restore_nocgi(sav_env, si, so):
     osenv = os.environ
 
     # restore env
-    for k in osenv.keys():
+    for k in list(osenv.keys()):
         del osenv[k]
     for k in sav_env:
         osenv[k] = sav_env[k]
@@ -924,18 +922,6 @@ def init(name, server):
 
     global _callback
     _callback = CallBack()
-
-    options = main_server.get_options()
-    #interpreters = options.get('mod_python.future.importer', None)
-    interpreters = options.get('mod_python.legacy.importer', None)
-    if interpreters:
-        interpreters = map(lambda x: x.strip(), interpreters.split(','))
-        #if '*' in interpreters or interpreter in interpreters:
-        if not ('*' in interpreters or interpreter in interpreters):
-            from mod_python import importer
-    else:
-        from mod_python import importer
-
     return _callback
 
 ## Some functions made public
@@ -986,6 +972,7 @@ HTTP_REQUEST_URI_TOO_LARGE        = 414
 HTTP_UNSUPPORTED_MEDIA_TYPE       = 415
 HTTP_RANGE_NOT_SATISFIABLE        = 416
 HTTP_EXPECTATION_FAILED           = 417
+HTTP_IM_A_TEAPOT                  = 418
 HTTP_UNPROCESSABLE_ENTITY         = 422
 HTTP_LOCKED                       = 423
 HTTP_FAILED_DEPENDENCY            = 424
@@ -1023,25 +1010,39 @@ except ImportError:
     APLOG_INFO = 6
     APLOG_DEBUG = 7
 
-APLOG_NOERRNO = 8
-
+APLOG_NOERRNO = 0 # DEPRECATED
 OK = REQ_PROCEED = 0
 DONE = -2
 DECLINED = REQ_NOACTION = -1
 
+_phase_handler_names = {}
+for _phase in ["PythonPostReadRequestHandler",
+              "PythonTransHandler",
+              "PythonHeaderParserHandler",
+              "PythonInitHandler",
+              "PythonAccessHandler",
+              "PythonAuthenHandler",
+              "PythonAuthzHandler",
+              "PythonTypeHandler",
+              "PythonFixupHandler",
+              "PythonHandler",
+              "PythonLogHandler",
+              "PythonCleanupHandler"]:
+    _phase_handler_names[_phase] = _phase[len("python"):].lower()
+
 _status_values = {
     "postreadrequesthandler":   [ DECLINED, OK ],
     "transhandler":             [ DECLINED ],
-    "maptostoragehandler":      [ DECLINED ],
-    "inithandler":              [ DECLINED, OK ],
     "headerparserhandler":      [ DECLINED, OK ],
+    "inithandler":              [ DECLINED, OK ],
     "accesshandler":            [ DECLINED, OK ],
     "authenhandler":            [ DECLINED ],
     "authzhandler":             [ DECLINED ],
     "typehandler":              [ DECLINED ],
     "fixuphandler":             [ DECLINED, OK ],
-    "loghandler":               [ DECLINED, OK ],
     "handler":                  [ OK ],
+    "loghandler":               [ DECLINED, OK ],
+    "cleanuphandler":           [ OK ],
 }
 
 # constants for get_remote_host
@@ -1054,7 +1055,6 @@ REMOTE_DOUBLE_REV = 3
 REQ_ABORTED = HTTP_INTERNAL_SERVER_ERROR
 REQ_EXIT = "REQ_EXIT"
 SERVER_RETURN = _apache.SERVER_RETURN
-PROG_TRACEBACK = "PROG_TRACEBACK"
 
 # the req.finfo tuple
 FINFO_MODE = 0
@@ -1148,15 +1148,15 @@ AP_MPMQ_MAX_REQUESTS_DAEMON= 11 # Max # of requests per daemon
 AP_MPMQ_MAX_DAEMONS        = 12 # Max # of daemons by config
 
 # magic mime types
-CGI_MAGIC_TYPE = "application/x-httpd-cgi" 
-INCLUDES_MAGIC_TYPE = "text/x-server-parsed-html" 
-INCLUDES_MAGIC_TYPE3 = "text/x-server-parsed-html3" 
-DIR_MAGIC_TYPE = "httpd/unix-directory" 
+CGI_MAGIC_TYPE = "application/x-httpd-cgi"
+INCLUDES_MAGIC_TYPE = "text/x-server-parsed-html"
+INCLUDES_MAGIC_TYPE3 = "text/x-server-parsed-html3"
+DIR_MAGIC_TYPE = "httpd/unix-directory"
 
 # for req.read_body
-REQUEST_NO_BODY = 0 
-REQUEST_CHUNKED_ERROR = 1 
-REQUEST_CHUNKED_DECHUNK = 2 
+REQUEST_NO_BODY = 0
+REQUEST_CHUNKED_ERROR = 1
+REQUEST_CHUNKED_DECHUNK = 2
 
 # for req.connection.keepalive
 AP_CONN_UNKNOWN = _apache.AP_CONN_UNKNOWN
@@ -1174,6 +1174,10 @@ APR_LNK = _apache.APR_LNK
 APR_SOCK = _apache.APR_SOCK
 APR_UNKFILE = _apache.APR_UNKFILE
 
+# module magic
+MODULE_MAGIC_NUMBER_MAJOR = _apache.MODULE_MAGIC_NUMBER_MAJOR
+MODULE_MAGIC_NUMBER_MINOR = _apache.MODULE_MAGIC_NUMBER_MINOR
+
 # for apache.stat()
 APR_FINFO_LINK = 0x00000001 # Stat the link not the file itself if it is a link
 APR_FINFO_MTIME = 0x00000010 # Modification Time
