@@ -1,5 +1,6 @@
  #
- # Copyright 2004 Apache Software Foundation
+ # Copyright (C) 2000, 2001, 2013, 2024 Gregory Trubetskoy
+ # Copyright (C) 2002, 2003, 2004, 2005, 2006, 2007 Apache Software Foundation
  #
  # Licensed under the Apache License, Version 2.0 (the "License"); you
  # may not use this file except in compliance with the License.  You
@@ -15,13 +16,12 @@
  #
  # Originally developed by Gregory Trubetskoy.
  #
- # $Id: python22.py 374268 2006-02-02 05:31:45Z nlehuen $
 
 # This file contains a bunch of hacks used to support Python 2.2
 
 import sys
 if sys.version < '2.3':
-    import __builtin__ as hack
-    
+    import builtins as hack
+
     # Enumerate does not exists in Python 2.2
-    hack.enumerate = lambda s : zip(xrange(len(s)),s)
+    hack.enumerate = lambda s : list(zip(list(range(len(s))),s))
