@@ -2,8 +2,9 @@
 #define Mp_APACHEMODULE_H
 
 /*
- * Copyright 2004 Apache Software Foundation 
- * 
+ * Copyright (C) 2000, 2001, 2013, 2024 Gregory Trubetskoy
+ * Copyright (C) 2002, 2003, 2004, 2005, 2006, 2007 Apache Software Foundation
+ *
  * Licensed under the Apache License, Version 2.0 (the "License"); you
  * may not use this file except in compliance with the License.  You
  * may obtain a copy of the License at
@@ -19,16 +20,14 @@
  * Originally developed by Gregory Trubetskoy.
  *
  *
- * apachemodule.h 
+ * apachemodule.h
  *
- * $Id: _apachemodule.h 106619 2004-11-25 22:10:52Z nd $
  *
- * See accompanying documentation and source code comments 
+ * See accompanying documentation and source code comments
  * for details.
  *
  */
 
 PyObject *get_ServerReturn(void);
-DL_EXPORT(void) init_apache(void);
 
 #endif /* !Mp_APACHEMODULE_H */
