@@ -1,6 +1,7 @@
 /*
- * Copyright 2004 Apache Software Foundation 
- * 
+ * Copyright (C) 2000, 2001, 2013, 2024 Gregory Trubetskoy
+ * Copyright (C) 2002, 2003, 2004, 2005, 2006, 2007 Apache Software Foundation
+ *
  * Licensed under the Apache License, Version 2.0 (the "License"); you
  * may not use this file except in compliance with the License.  You
  * may obtain a copy of the License at
@@ -18,7 +19,6 @@
  *
  * requestobject.h
  *
- * $Id: requestobject.h 102649 2004-02-16 19:47:28Z grisha $
  *
  */
 
@@ -34,29 +34,34 @@ extern "C" {
         request_rec    * request_rec;
         PyObject       * connection;
         PyObject       * server;
-        PyObject       * next;
-        PyObject       * prev;
-        PyObject       * main;
         PyObject       * headers_in;
         PyObject       * headers_out;
         PyObject       * err_headers_out;
         PyObject       * subprocess_env;
         PyObject       * notes;
         PyObject       * phase;
+        PyObject       * config;
+        PyObject       * options;
         char           * extension;   /* for | .ext syntax */
-        char           * interpreter; 
         int              content_type_set;
+        apr_off_t        bytes_queued;
         hlistobject    * hlo;
         char           * rbuff;       /* read bufer */
         int              rbuff_len;   /* read buffer size */
         int              rbuff_pos;   /* position into the buffer */
+        PyObject       * session;
+
     } requestobject;
 
-    extern DL_IMPORT(PyTypeObject) MpRequest_Type;
-    
-#define MpRequest_Check(op) ((op)->ob_type == &MpRequest_Type)
-    
-    extern DL_IMPORT(PyObject *) MpRequest_FromRequest Py_PROTO((request_rec *r));
+    PyAPI_DATA(PyTypeObject) MpRequest_Type;
+
+#define MpRequest_Check(op) (Py_TYPE(op) == &MpRequest_Type)
+
+    PyAPI_FUNC(PyObject *) MpRequest_FromRequest (request_rec *r);
+
+#ifndef ap_is_HTTP_VALID_RESPONSE
+#define ap_is_HTTP_VALID_RESPONSE(x) (((x) >= 100)&&((x) < 600))
+#endif
 
 #ifdef __cplusplus
 }
