@@ -2,8 +2,9 @@
 #define Mp_UTIL_H
 
 /*
- * Copyright 2004 Apache Software Foundation 
- * 
+ * Copyright (C) 2000, 2001, 2013, 2024 Gregory Trubetskoy
+ * Copyright (C) 2002, 2003, 2004, 2005, 2006, 2007 Apache Software Foundation
+ *
  * Licensed under the Apache License, Version 2.0 (the "License"); you
  * may not use this file except in compliance with the License.  You
  * may obtain a copy of the License at
@@ -19,11 +20,10 @@
  * Originally developed by Gregory Trubetskoy.
  *
  *
- * util.h 
+ * util.h
  *
- * $Id: util.h 106619 2004-11-25 22:10:52Z nd $
  *
- * See accompanying documentation and source code comments 
+ * See accompanying documentation and source code comments
  * for details.
  *
  */
@@ -36,5 +36,6 @@ char * get_addhandler_extensions(request_rec *req);
 apr_status_t python_decref(void *object);
 PyMemberDef *find_memberdef(const PyMemberDef *mlist, const char *name);
 PyObject *cfgtree_walk(ap_directive_t *dir);
+PyObject *makesockaddr(struct apr_sockaddr_t *addr);
 
 #endif /* !Mp_UTIL_H */
