@@ -1,6 +1,7 @@
 /*
- * Copyright 2004 Apache Software Foundation 
- * 
+ * Copyright (C) 2000, 2001, 2013, 2024 Gregory Trubetskoy
+ * Copyright (C) 2002, 2003, 2004, 2005, 2006, 2007 Apache Software Foundation
+ *
  * Licensed under the Apache License, Version 2.0 (the "License"); you
  * may not use this file except in compliance with the License.  You
  * may obtain a copy of the License at
@@ -16,9 +17,8 @@
  * Originally developed by Gregory Trubetskoy.
  *
  *
- * serverobject.c 
+ * serverobject.c
  *
- * $Id: serverobject.c 481717 2006-12-03 04:36:37Z grahamd $
  *
  */
 
@@ -61,7 +61,7 @@ PyObject * MpServer_FromServer(server_rec *s)
 static PyObject * server_get_config(serverobject *self)
 {
     py_config *conf =
-        (py_config *) ap_get_module_config(self->server->module_config, 
+        (py_config *) ap_get_module_config(self->server->module_config,
                                            &python_module);
     return MpTable_FromTable(conf->directives);
 }
@@ -98,7 +98,7 @@ static PyObject * server_log_error(serverobject *self, PyObject *args)
     if (message) {
 
         if (! level)
-            level = APLOG_NOERRNO|APLOG_ERR;
+            level = APLOG_ERR;
 
         ap_log_error(APLOG_MARK, level, 0, self->server, "%s", message);
     }
@@ -124,30 +124,39 @@ static PyObject *server_register_cleanup(serverobject *self, PyObject *args)
     requestobject *req = NULL;
     PyObject *name_obj = NULL;
     char *name = NULL;
+    char *c_name_obj;
 
     if (! PyArg_ParseTuple(args, "OO|O", &req, &handler, &data))
-        return NULL; 
+        return NULL;
 
     if (! MpRequest_Check(req)) {
-        PyErr_SetString(PyExc_ValueError, 
+        PyErr_SetString(PyExc_ValueError,
                         "first argument must be a request object");
         return NULL;
     }
     else if(!PyCallable_Check(handler)) {
-        PyErr_SetString(PyExc_ValueError, 
+        PyErr_SetString(PyExc_ValueError,
                         "second argument must be a callable object");
         return NULL;
     }
-    
+
     ci = (cleanup_info *)malloc(sizeof(cleanup_info));
     ci->request_rec = NULL;
     ci->server_rec = self->server;
     Py_INCREF(handler);
     ci->handler = handler;
     name_obj = python_interpreter_name();
-    name = (char *)malloc(strlen(PyString_AsString(name_obj))+1);
-    strcpy(name, PyString_AsString(name_obj));
+    MP_ANYSTR_AS_STR(c_name_obj, name_obj, 1);
+    if (!c_name_obj) {
+        Py_DECREF(name_obj); /* MP_ANYSTR_AS_STR */
+        return NULL;
+    }
+    name = (char *)malloc(strlen(c_name_obj)+1);
+    if (!name)
+        return PyErr_NoMemory();
+    strcpy(name, c_name_obj);
     ci->interpreter = name;
+    Py_DECREF(name_obj); /* MP_ANYSTR_AS_STR */
     if (data) {
         Py_INCREF(data);
         ci->data = data;
@@ -156,10 +165,10 @@ static PyObject *server_register_cleanup(serverobject *self, PyObject *args)
         Py_INCREF(Py_None);
         ci->data = Py_None;
     }
-    
-    apr_pool_cleanup_register(child_init_pool, ci, python_cleanup, 
+
+    apr_pool_cleanup_register(child_init_pool, ci, python_cleanup,
                               apr_pool_cleanup_null);
-    
+
     Py_INCREF(Py_None);
     return Py_None;
 }
@@ -173,7 +182,7 @@ static PyMethodDef server_methods[] = {
 };
 
 
-/* 
+/*
    These are offsets into the Apache server_rec structure.
    They are accessed via getset functions. Note that the types
    specified here are irrelevant if a function other than
@@ -184,14 +193,18 @@ static PyMethodDef server_methods[] = {
 
 #define OFF(x) offsetof(server_rec, x)
 
-static struct PyMemberDef server_rec_mbrs[] = {
+static PyMemberDef server_rec_mbrs[] = {
     {"defn_name",          T_STRING,  OFF(defn_name)},
     {"defn_line_number",   T_INT,     OFF(defn_line_number)},
     {"server_admin",       T_STRING,  OFF(server_admin)},
     {"server_hostname",    T_STRING,  OFF(server_hostname)},
     {"port",               T_SHORT,   OFF(port)},
     {"error_fname",        T_STRING,  OFF(error_fname)},
+#if AP_MODULE_MAGIC_AT_LEAST(20100606,0)
+    {"loglevel",           T_INT,     OFF(log.level)},
+#else
     {"loglevel",           T_INT,     OFF(loglevel)},
+#endif
     {"is_virtual",         T_INT,     OFF(is_virtual)},
     /* XXX implement module_config ? */
     /* XXX implement lookup_defaults ? */
@@ -201,14 +214,14 @@ static struct PyMemberDef server_rec_mbrs[] = {
     {"keep_alive_max",     T_INT,     OFF(keep_alive_max)},
     {"keep_alive",         T_INT,     OFF(keep_alive)},
     /* XXX send_buffer_size gone. where? document */
-    /*{"send_buffer_size",   T_INT,       OFF(send_buffer_size),   RO},*/
+    /*{"send_buffer_size",   T_INT,       OFF(send_buffer_size),   READONLY},*/
     {"path",               T_STRING,  OFF(path)},
     {"pathlen",            T_INT,     OFF(pathlen)},
     {"names",              T_OBJECT,  OFF(names)},
     {"wild_names",         T_OBJECT,  OFF(wild_names)},
     /* XXX server_uid and server_gid seem gone. Where? Document. */
-    /*{"server_uid",         T_INT,       OFF(server_uid),         RO},*/
-    /*{"server_gid",         T_INT,       OFF(server_gid),         RO},*/
+    /*{"server_uid",         T_INT,       OFF(server_uid),         READONLY},*/
+    /*{"server_gid",         T_INT,       OFF(server_gid),         READONLY},*/
     /* XXX Document limit* below. Make RW? */
     {"limit_req_line",       T_INT,   OFF(limit_req_line)},
     {"limit_req_fieldsize",  T_INT,   OFF(limit_req_fieldsize)},
@@ -222,13 +235,23 @@ static struct PyMemberDef server_rec_mbrs[] = {
  *    Retrieves server_rec structure members
  */
 
-static PyObject *getsrv_recmbr(serverobject *self, void *name) 
+static PyObject *getsrv_recmbr(serverobject *self, void *name)
 {
     if (strcmp(name, "_server_rec") == 0) {
+#if PY_MAJOR_VERSION == 2 && PY_MINOR_VERSION < 7
         return PyCObject_FromVoidPtr(self->server, 0);
+#else
+        return PyCapsule_New((void *)self->server, NULL, NULL);
+#endif
+
     }
-    return PyMember_GetOne((char*)self->server,
-                           find_memberdef(server_rec_mbrs, name));
+
+    PyMemberDef *md = find_memberdef(server_rec_mbrs, name);
+    if (!md) {
+        PyErr_SetString(PyExc_AttributeError, name);
+        return NULL;
+    }
+    return PyMember_GetOne((char*)self->server, md);
 }
 
 /* we don't need setsrv_recmbr for now */
@@ -239,7 +262,7 @@ static PyObject *getsrv_recmbr(serverobject *self, void *name)
  *    Retrieves apr_time_t server_rec members
  */
 
-static PyObject *getsrv_recmbr_time(serverobject *self, void *name) 
+static PyObject *getsrv_recmbr_time(serverobject *self, void *name)
 {
     PyMemberDef *md = find_memberdef(server_rec_mbrs, name);
     char *addr = (char *)self->server + md->offset;
@@ -253,10 +276,10 @@ static PyObject *getsrv_recmbr_time(serverobject *self, void *name)
  *    For array headers that will get converted to tuple
  */
 
-static PyObject *getsrv_recmbr_ah(serverobject *self, void *name) 
+static PyObject *getsrv_recmbr_ah(serverobject *self, void *name)
 {
     const PyMemberDef *md = find_memberdef(server_rec_mbrs, name);
-    apr_array_header_t *ah = 
+    apr_array_header_t *ah =
         *(apr_array_header_t **)((char *)self->server + md->offset);
 
     return tuple_from_array_header(ah);
@@ -268,7 +291,7 @@ static PyObject *getsrv_recmbr_ah(serverobject *self, void *name)
  *    A getter func that creates an object as needed.
  */
 
-static PyObject *getmakeobj(serverobject* self, void *objname) 
+static PyObject *getmakeobj(serverobject* self, void *objname)
 {
     char *name = (char *)objname;
     PyObject *result = NULL;
@@ -288,7 +311,13 @@ static PyObject *getmakeobj(serverobject* self, void *objname)
 
 static PyObject *my_generation(serverobject *self, void *objname)
 {
-    return PyInt_FromLong((long)ap_my_generation);
+    int mpm_generation = 0;
+#if defined(AP_MPMQ_GENERATION)
+    ap_mpm_query(AP_MPMQ_GENERATION, &mpm_generation);
+#else
+    mpm_generation = ap_my_generation;
+#endif
+    return PyLong_FromLong((long)mpm_generation);
 }
 
 static PyObject *restart_time(serverobject *self, void *objname)
@@ -300,7 +329,7 @@ static PyGetSetDef server_getsets[] = {
     /* XXX process */
     {"next",         (getter)getmakeobj,    NULL, "The next server in the list", "next"},
     {"defn_name",    (getter)getsrv_recmbr, NULL, "The name of the server", "defn_name"},
-    {"defn_line_number",    (getter)getsrv_recmbr, NULL, 
+    {"defn_line_number",    (getter)getsrv_recmbr, NULL,
           "The line of the config file that the server was defined on", "defn_line_number"},
     {"server_admin", (getter)getsrv_recmbr, NULL, "The admin's contact information", "server_admin"},
     {"server_hostname",    (getter)getsrv_recmbr, NULL, "The server hostname", "server_hostname"},
@@ -333,8 +362,9 @@ static PyGetSetDef server_getsets[] = {
  *
  */
 
-static void server_dealloc(serverobject *self)
-{  
+static void server_dealloc(void *o)
+{
+    serverobject *self = (serverobject *)o;
     Py_XDECREF(self->dict);
     Py_XDECREF(self->next);
     PyObject_Del(self);
@@ -344,21 +374,20 @@ static char server_doc[] =
 "Apache server_rec structure\n";
 
 PyTypeObject MpServer_Type = {
-    PyObject_HEAD_INIT(NULL)
-    0,
-    "mp_server",
-    sizeof(serverobject),
-    0,
-    (destructor) server_dealloc,     /*tp_dealloc*/
-    0,                               /*tp_print*/
-    0,                               /*tp_getattr*/
-    0,                               /*tp_setattr*/
-    0,                               /*tp_compare*/
-    0,                               /*tp_repr*/
-    0,                               /*tp_as_number*/
-    0,                               /*tp_as_sequence*/
-    0,                               /*tp_as_mapping*/
-    0,                               /*tp_hash*/
+    PyVarObject_HEAD_INIT(&PyType_Type, 0)
+    "mp_server",                     /* tp_name */
+    sizeof(serverobject),            /* tp_basicsize */
+    0,                               /* tp_itemsize */
+    (destructor) server_dealloc,     /* tp_dealloc*/
+    0,                               /* tp_print*/
+    0,                               /* tp_getattr*/
+    0,                               /* tp_setattr*/
+    0,                               /* tp_compare*/
+    0,                               /* tp_repr*/
+    0,                               /* tp_as_number*/
+    0,                               /* tp_as_sequence*/
+    0,                               /* tp_as_mapping*/
+    0,                               /* tp_hash*/
     0,                               /* tp_call */
     0,                               /* tp_str */
     PyObject_GenericGetAttr,         /* tp_getattro */
@@ -384,10 +413,5 @@ PyTypeObject MpServer_Type = {
     0,                               /* tp_init */
     0,                               /* tp_alloc */
     0,                               /* tp_new */
-    (destructor)server_dealloc,      /* tp_free */
+    server_dealloc,                  /* tp_free */
 };
-
-
-
-
-
